@@ -36,8 +36,6 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <errno.h>
-#define ALSA_PCM_NEW_HW_PARAMS_API
-#define ALSA_PCM_NEW_SW_PARAMS_API
 #include <alsa/asoundlib.h>
 #include <assert.h>
 #include <sys/poll.h>
@@ -201,12 +199,12 @@ static void device_list(void)
 		sprintf(name, "hw:%d", card);
 		if ((err = snd_ctl_open(&handle, name, 0)) < 0) {
 			error("control open (%i): %s", card, snd_strerror(err));
-			continue;
+			goto next_card;
 		}
 		if ((err = snd_ctl_card_info(handle, info)) < 0) {
 			error("control hardware info (%i): %s", card, snd_strerror(err));
 			snd_ctl_close(handle);
-			continue;
+			goto next_card;
 		}
 		fprintf(stderr, "**** List of %s Hardware Devices ****\n", snd_pcm_stream_name(stream));
 		dev = -1;
@@ -241,6 +239,7 @@ static void device_list(void)
 			}
 		}
 		snd_ctl_close(handle);
+	next_card:
 		if (snd_card_next(&card) < 0) {
 			error("snd_card_next");
 			break;
@@ -1514,11 +1513,15 @@ static off64_t calc_count(void)
 {
 	off64_t count;
 
-	if (!timelimit) {
+	if (!timelimit && pbrec_count == (size_t)-1) {
 		count = (off64_t)-1;
 	} else {
-		count = snd_pcm_format_size(hwparams.format, hwparams.rate * hwparams.channels);
-		count *= (off64_t)timelimit;
+		if (timelimit == 0) {
+			count = pbrec_count;
+		} else {
+			count = snd_pcm_format_size(hwparams.format, hwparams.rate * hwparams.channels);
+			count *= (off64_t)timelimit;
+		}
 	}
 	return count < pbrec_count ? count : pbrec_count;
 }
